#include "host.h"
#include "clientprot.h"
#include "math.h"
#include "fileio.h"


//////// Host messaging ////////

hostMessage::hostMessage(void * nmsg, Uint32 nlen, Host * nsrc)
{
	len = nlen;
	src = nsrc;
	msg = new char[len];
	MEMCPY(nmsg, msg, len);
}

hostMessage::~hostMessage()
{
	delete []msg;
}

BYTE hostMessage::getType(bool special)
{
	if (special)
		return msg[1];
	else
		return msg[0];
}


//////// Cluster message ////////

clusterMessage::clusterMessage(char * nmsg, Uint32 nlen)
{
	msg = new char[len = nlen];
	MEMCPY(nmsg, msg, len);
}

clusterMessage::~clusterMessage()
{
	delete []msg;
}


//////// Client message ////////

clientMessage::clientMessage(Uint32 nlen)
{
	msg = new char[len = nlen];
}

clientMessage::~clientMessage()
{
	delete []msg;
}


//////// Reliable message ////////

reliableMessage::reliableMessage(Uint32 nACK_ID, char * nmsg, Uint32 nlen)
{
	msg = new char[len = nlen];
	MEMCPY(nmsg, msg, len);
	ACK_ID = nACK_ID;
	setTime();
}

reliableMessage::~reliableMessage()
{
	delete []msg;
}

void reliableMessage::setTime()
{
	lastSend = GetTickCount() / 10;
}


//////// Clustering ////////

void Host::beginCluster()
{
	if (CLUSTER_MODE) clustering = true;
}

void Host::post(char * msg, Uint32 len, bool reliable)
{
	if (len > CHUNK_SIZE + 12)
	{	// Chunk it
		if (len > 1000)
		{
			for (Uint32 i = 0; i < len; i += CHUNK_SIZE)
			{
				char * buffer = new char[CHUNK_SIZE + 6];
				buffer[0] = 0x00;
				buffer[1] = 0x0A;
				*(Uint32*)&buffer[2] = len;

				Uint32 remaining = (len - i);

				if (remaining > CHUNK_SIZE)
				{	// Chunk body
					MEMCPY(msg + i, buffer + 6, CHUNK_SIZE);

					post(buffer, CHUNK_SIZE + 6, true);
				}
				else
				{	// Chunk tail
					MEMCPY(msg + i, buffer + 6, remaining);

					post(buffer, remaining + 6, true);
				}

				delete []buffer;
			}
		}
		else
		{
			for (Uint32 i = 0; i < len; i += (CHUNK_SIZE + 4))
			{
				char * buffer = new char[CHUNK_SIZE + 6];
				buffer[0] = 0x00;

				Uint32 remaining = (len - i);

				if (remaining > (CHUNK_SIZE + 4))
				{	// Chunk body
					buffer[1] = 0x08;

					MEMCPY(msg + i, buffer + 2, CHUNK_SIZE + 4);

					post(buffer, CHUNK_SIZE + 6, true);
				}
				else
				{	// Chunk tail
					buffer[1] = 0x09;

					MEMCPY(msg + i, buffer + 2, remaining);

					post(buffer, remaining + 2, true);
				}

				delete []buffer;
			}
		}
	}
	else
	{	// We're dealing with byte-size messages now
		char buffer[520];

		if (reliable)
		{	// Prepend allocated reliable header
			buffer[0] = 0x00;
			buffer[1] = 0x03;
			*(Uint32*)&buffer[2] = localStep;
			MEMCPY(msg, buffer + 6, len);
			msg = buffer;
			len += 6;

			if (queue(msg, len) == false) return;
		}

		if (clustering)
		{	// Append to cluster list
			clusterMessage * m = new clusterMessage(msg, len);
			if (m)
			{
				clustered.append(m);
			}
			else
			{
				send(msg, len);
			}
		}
		else
		{	// Send right away
			send(msg, len);
		}
	}
}

void Host::sendClustered(_listnode <clusterMessage> * parse)
{
	char buffer[520];
	buffer[0] = 0x00;
	buffer[1] = 0x0E;
	Uint32 len = 2;		// Total length of the cluster message
	Uint32 tctr = 0;	// Total messages clustered

	while (parse)
	{
		clusterMessage * m = parse->item;

		if (m->len > 255)
		{	// This message will not fit on a cluster
			send(m->msg, m->len);
		}
		else
		{	// This message will fit on a cluster
			if (len + m->len > CHUNK_SIZE + 12)
			{	// But, this cluster message is full
				if (tctr == 1)
				{
					send(buffer + 3, len - 3);
				}
				else
				{
					send(buffer, len);
				}

				sendClustered(parse->next);

				return;
			}

			// Append the next listed message
			buffer[len++] = (char)m->len;
			MEMCPY(m->msg, buffer + len, m->len);
			len += m->len;
			tctr++;
		}

		parse = parse->next;
	}

	// Send the end of the recursive tree
	switch (tctr)
	{
	case 0:
		break;
	case 1:
		send(buffer + 3, len - 3);
		break;
	default:
		send(buffer, len);
	};
}

void Host::endCluster()
{
	clustering = false;

	sendClustered(clustered.head);

	clustered.clear(false);
}


//////// Ping math ////////

void Host::resetPingTable()
{
	stdDev = 0;
	tableIndex = 0;
	for (int i = 0; i < 32; i++)
	{
		pingTable[i] = -1;
	}
}

void Host::addPing(Uint32 time)
{
	// Add an entry
	pingTable[tableIndex++] = time;
	if (tableIndex == 32) tableIndex = 0;

	// Update low/high stats
	if ((highPing == 0) || (time > highPing))
	{
		highPing = time;
	}
	else if ((lowPing == 0) || (time < lowPing))
	{
		lowPing = time;
	}

	// Calculate the mean & total
	Uint32 mean = 0;
	for (int i = 0; i < 32; i++)
	{
		if (pingTable[i] == -1)
			break;

		mean += pingTable[i];
	}
	mean /= i + 1;

	avgPing = mean;

	// Calculate the standard deviation
	Uint32 mul, sd = 0;
	for (int j = 0; j < i; j++)
	{
		mul = pingTable[j] - mean;
		sd += mul * mul;
	}
	sd = sqrt(sd / i);	// Unbiased

	stdDev = sd + 1;
}


//////// Reliable messaging ////////

bool Host::queue(char * msg, Uint32 len)
{	// Returns false if the message shouldn't be sent immediately
	reliableMessage * m = new reliableMessage(getLong(msg, 2), msg, len);

	localStep++;	// Increment next ACK_ID

	if ((Uint32)sent.total < MAX_RELIABLE_IN_TRANSIT)
	{	// There is a free send slot
		sent.append(m);
		++localSent;	// Increment most recently sent ACK_ID

		return true;
	}
	else
	{	// It must be queued for delivery
		queued.append(m);

		return false;
	}
}

void Host::checkSent(Uint32 ACK_ID)
{
	// Ignore if we haven't sent it yet
	if (ACK_ID > localSent)
	{
		return;
	}

	// Stop sending if an ACK was recv'd
	_listnode <reliableMessage> * parse = sent.head;

	while (parse)
	{
		reliableMessage * m = parse->item;

		if (m->ACK_ID == ACK_ID)
		{
/*			Uint32 ping = getTime() - m->lastSend;
			if (ping > syncPing)
			{
				S2CSlowCurrent++;
			}
			else
			{
				S2CFastCurrent++;
			}
*/
			sent.kill(parse, false);
			sendNext();

			return;
		}

		parse = parse->next;
	}
}

void Host::sendNext()
{
	// Queue up another packet for sending
	_listnode <reliableMessage> * parse = queued.head;

	while (parse)
	{
		reliableMessage * m = parse->item;

		if (m->ACK_ID == localSent)
		{
			// Send it
			post(m->msg, m->len, false);
			m->setTime();

			// Move the message
			sent.append(m);
			++localSent;	// Increment most recently sent ACK_ID
			queued.kill(parse, true);

			return;
		}

		parse = parse->next;
	}
}

void Host::checkReceived(Uint32 ACK_ID, char * msg, Uint32 len)
{
	// Ignore if we've already processed it
	if (ACK_ID < remoteStep)
	{
		return;
	}

	// Poke around and find a match
	if (ACK_ID == remoteStep)
	{
		gotMessage(msg, len);
		++remoteStep;

		// Process next in sequence
		_listnode <reliableMessage> * parse = received.head;

		while (parse)
		{
			reliableMessage * recv = parse->item;

			if (recv->ACK_ID == remoteStep)
			{
				gotMessage(recv->msg, recv->len);
				++remoteStep;
				received.kill(parse, false);

				parse = received.head;
			}
			else
			{
				parse = parse->next;
			}
		}
	}
	else
	{
		// Do not add if it already exists
		_listnode <reliableMessage> * parse = received.head;

		while (parse)
		{
			reliableMessage * recv = parse->item;

			if (recv->ACK_ID == ACK_ID)
				return;

			parse = parse->next;
		}

		// Add it
		reliableMessage * rm = new reliableMessage(ACK_ID, msg, len);
		if (rm != NULL)
		{
			received.append(rm);
		}
	}
}

void Host::sendACK(Uint32 ACK_ID)
{
	char msg[6];
	msg[0] = 0x00;
	msg[1] = 0x04;
	*(Uint32*)&msg[2] = ACK_ID;

	post(msg, 6, false);
}

void Host::sendQueued()
{
	_listnode <reliableMessage> * parse = sent.head;

	Uint32 time = getTime();

	while (parse)
	{
		reliableMessage * m = parse->item;

		if (time - m->lastSend > syncPing + stdDev)
		{
			post(m->msg, m->len, false);
			m->setTime();
		}

		parse = parse->next;
	}
}


//////// Host messaging ////////

void Host::send(char * msg, Uint32 len)
{
	char * packet = new char[len];
	MEMCPY(msg, packet, len);

	encryption.encrypt(packet, len);

	socket->set(remote);
	socket->send(packet, len);

	delete []packet;
	msgSent++;
}


//////// Chat ////////

void Host::sendPrivate(Player * player, BYTE snd, char * msg)
{
	clientMessage * cm = generateChat(MSG_Private, (Chat_SoundBytes)snd, player->ident, msg);
	post(cm->msg, cm->len, true);
	delete cm;
}

void Host::sendPrivate(Player * player, char * msg)
{
	clientMessage * cm = generateChat(MSG_Private, SND_None, player->ident, msg);
	post(cm->msg, cm->len, true);
	delete cm;
}


//////// Game protocol ////////

Uint32 Host::getHostTime()
{
	return (getTime() + timeDiff);
}

Uint32 Host::getLocalTime(Uint32 time)
{
	return (time - timeDiff);
}

Uint32 Host::getTime()
{
	return (GetTickCount() / 10);
}


//////// Core protocol ////////

Host::Host(bool protoSide, INADDR * hostName, UDPSocket * nSocket)
{
	// Winsock
	remote = hostName;
	socket = nSocket;

	for (int i = 0; i < 256; i++)
	{
		specialRouter.add(i, handleSpecialUnknown);
		generalRouter.add(i, handleUnknown);
	}

	// Routers
	generalRouter.add(0x00, handleSpecialHeader);
	if (serverSide = protoSide)
	{
		specialRouter.add(0x01, handleEncryptRequest);
	}
	else
	{
		specialRouter.add(0x05, handleSyncRequest);
		specialRouter.add(0x02, handleEncryptResponse);
	}
	specialRouter.add(0x07, handleDisconnect);

	// Transfer mode
	ftLength = 0;
	little.setLimit(5000);		// 5KB - enough for a server list
	large.setLimit(1000000);	// 1MB - enough for your average file transfer
								// There is NOTHING more than 1 MB in a SubSpace server folder
	connecting = false;
	syncing = false;
	killMe = false;
	lastRecv = getTime();

	// Sync
	timeDiff = 0;
	msgSent = 0;
	msgRecv = 0;
	syncPing = 100;
	resetPingTable();
	highPing = 0;
	lowPing = 0;

	// Reliables
	remoteStep = 0;
	localStep = 0;
	localSent = 0;
	S2CSlowCurrent = 0;
	S2CFastCurrent = 0;
	S2CSlowTotal = 0;
	S2CFastTotal = 0;

	// Arena
	resetArena();

	fh = _open("log.txt", _O_CREAT | _O_RDWR | _O_APPEND | _O_TEXT, _S_IREAD | _S_IWRITE);
}

Host::~Host()
{
	if (killMe == false)
		disconnect(true);

	_close(fh);
}

void Host::resetArena()
{
	playerlist.clear(false);
	qalist.clear();

	position = false;

	ps.clearList();

	Me = -1;
}

bool Host::validateSource(INADDR * src)
{
	return ((src->ip == remote.ip) && (src->port == remote.port));
}

void Host::doEvents()
{
	beginCluster();

		if (getTime() - lastRecv > PACKET_SILENCE_LIMIT)
		{
			disconnect(true);
		}

		sendQueued();

		if (connecting)
		{
			if (getTime() - lastConnect > ((syncPing + stdDev) << 4))
				connect();
		}

		if (syncing)
		{
			if (getTime() - lastSync > syncPing + stdDev)
				syncClocks();
		}

		if (position)
		{
			if (getTime() - lastPosition > 5)
			{
				Player * p = getPlayer(Me);

				if (p)
				{
					lastPosition = getTime();
					clientMessage * cm = generatePosition(getHostTime(),
														  p->d,
														  p->x,
														  p->y,
														  p->xvel,
														  p->yvel,
														  p->stealth,
														  p->cloak,
														  p->xradar,
														  p->awarp,
														  false,
														  false,
														  p->ufo,
														  p->bounty,
														  p->energy,
														  PROJ_None, 0, false, false, false, false, false,
														  p->timer,
														  (Uint16)syncPing,
														  p->shields,
														  p->super,
														  p->burst,
														  p->repel,
														  p->thor,
														  p->brick,
														  p->decoy,
														  p->rocket,
														  p->portal);
					post(cm->msg, cm->len, false);
					delete cm;
				}
			}
		}

	endCluster();

	ps.doPrizes((Uint16)playerlist.total);
	doBrickEvents();
}

void Host::gotPacket(char * msg, Uint32 len)
{
	encryption.decrypt(msg, len);

	beginCluster();
		gotMessage(msg, len);
	// endCluster() gets called by hostList through Host::doEvents()

	msgRecv++;	// Note: do not recursively call gotPacket()
	lastRecv = getTime();
}

void Host::gotMessage(char * msg, Uint32 len)
{
	hostMessage * m = new hostMessage(msg, len, this);
	if (m)
	{
		gotMessage(m);

		delete m;
	}
}

void Host::gotMessage(hostMessage * msg)
{
	generalRouter.call(msg->getType(false), msg);
}

void Host::gotSpecialMessage(hostMessage * msg)
{
	specialRouter.call(msg->getType(true), msg);
}

void Host::gotEncryptRequest(Uint32 key, Uint16 prot)
{
	// Force SubSpace encryption
	Uint32 sessionKey = encryption.getSessionKey(key);

	// Stuff we won't be needing anymore
	specialRouter.kill(0x01);

	// Reliable messaging
	specialRouter.add(0x03, handleReliable);
	specialRouter.add(0x04, handleACK);

	// Synchronization
	specialRouter.add(0x05, handleSyncRequest);

	// Transmission types
	specialRouter.add(0x08, handleChunkBody);
	specialRouter.add(0x09, handleChunkTail);
	specialRouter.add(0x0A, handleBigChunk);
	specialRouter.add(0x0E, handleCluster);

	// Start encryption
	char msg[6];
	msg[0] = 0x00;
	msg[1] = 0x02;
	*(Uint32*)&msg[2] = sessionKey;
	send(msg, 6);

	encryption.initializeEncryption(sessionKey);

	// Game protocol
}

void Host::gotEncryptResponse(Uint32 key)
{
	connecting = false;

	if (encryption.validateSessionKey(key))
	{
		encryption.initializeEncryption(key);

		// Stuff we won't be needing anymore
		specialRouter.kill(0x05);
		specialRouter.kill(0x02);

		// Reliable messaging
		specialRouter.add(0x03, handleReliable);
		specialRouter.add(0x04, handleACK);

		// Synchronization
		specialRouter.add(0x06, handleSyncResponse);
		syncClocks();

		// Transmission types
		specialRouter.add(0x08, handleChunkBody);
		specialRouter.add(0x09, handleChunkTail);
		specialRouter.add(0x0A, handleBigChunk);
		specialRouter.add(0x0E, handleCluster);

		// Game protocol
		generalRouter.add(0x0A,	handlePasswordResponse);
		generalRouter.add(0x10,	handleFileTransfer);
		generalRouter.add(0x31,	handleLoginNext);

		// Additional Continuum game protocol
		generalRouter.add(0x33,	handleCustomMessage);

		printf("Sending password for %s\n", botInfo.name);
		clientMessage * cm = generatePassword(false, botInfo.name, botInfo.password, botInfo.machineID, botInfo.timeZoneBias, botInfo.permissionID, botInfo.clientVersion);
		post(cm->msg, cm->len, true);
		delete cm;
	}
}

void Host::activateGameProtocol()
{
	// Enabled protocol
	generalRouter.add(0x01,	handleIdent);
	generalRouter.add(0x02,	handleInGameFlag);
	generalRouter.add(0x03,	handlePlayerEntering);
	generalRouter.add(0x04,	handlePlayerLeaving);
	generalRouter.add(0x05,	handleWeaponUpdate);
	generalRouter.add(0x06,	handlePlayerDeath);
	generalRouter.add(0x07,	handleChat);
	generalRouter.add(0x08,	handlePlayerPrize);
	generalRouter.add(0x09,	handleScoreUpdate);
	generalRouter.add(0x0B,	handleSoccerGoal);
	generalRouter.add(0x0D,	handleSetTeam);
	generalRouter.add(0x0E,	handleCreateTurret);
	generalRouter.add(0x0F,	handleArenaSettings);
	generalRouter.add(0x12,	handleFlagPosition);
	generalRouter.add(0x13,	handleFlagClaim);
	generalRouter.add(0x14,	handleFlagVictory);
	generalRouter.add(0x15,	handleDeleteTurret);
	generalRouter.add(0x16,	handleFlagDrop);
	generalRouter.add(0x18,	handleSecurity);
	generalRouter.add(0x19,	handleFileRequest);
	generalRouter.add(0x1A,	handleScoreReset);
	generalRouter.add(0x1B,	handleShipReset);
	generalRouter.add(0x1C,	handleSpecPlayer);
	generalRouter.add(0x1D,	handleSetTeamAndShip);
	generalRouter.add(0x1E,	handleBannerFlag);
	generalRouter.add(0x1F,	handlePlayerBanner);
	generalRouter.add(0x20,	handleSelfPrize);
	generalRouter.add(0x21,	handleBrickDrop);
	generalRouter.add(0x22,	handleTurfFlagStatus);
	generalRouter.add(0x23,	handleFlagReward);
	generalRouter.add(0x25,	handleToggleUFO);
	generalRouter.add(0x27,	handleKeepAlive);
	generalRouter.add(0x28,	handlePlayerPosition);
	generalRouter.add(0x29,	handleMapInfo);
	generalRouter.add(0x2A,	handleMapFile);
	generalRouter.add(0x2C,	handleKoTHReset);
	generalRouter.add(0x2E,	handleBallPosition);
	generalRouter.add(0x2F,	handleArenaList);
	generalRouter.add(0x30,	handleBannerAds);
	generalRouter.add(0x32,	handleChangePosition);

	// Disabled protocol
	generalRouter.kill(0x0A);	// Password response
	generalRouter.kill(0x31);	// Login next
	generalRouter.kill(0x33);	// Custom response
}

void Host::gotEncryptResponse(Uint32 key, BYTE mudge)
{
	gotEncryptResponse(key);
}

void Host::gotReliable(Uint32 id, char * msg, Uint32 len)
{
	sendACK(id);
	checkReceived(id, msg, len);
}

void Host::gotACK(Uint32 id)
{
	checkSent(id);
}

void Host::gotSyncRequest(Uint32 time)
{
	char msg[10];
	msg[0] = 0x00;
	msg[1] = 0x06;
	*(Uint32*)&msg[2] = time;
	*(Uint32*)&msg[6] = getTime();

	post(msg, 10, false);
}

void Host::gotSyncRequest(Uint32 time, Uint32 sent, Uint32 recv)
{
	gotSyncRequest(time);
}

void Host::gotSyncResponse(Uint32 pingTime, Uint32 pongTime)
{
	timeDiff = pongTime - pingTime;
	syncPing = getTime() - pingTime;
	addPing(syncPing);
	syncing = false;
}

void Host::gotDisconnect()
{
	disconnect(false);
}

void Host::gotChunkBody(char * msg, Uint32 len)
{
	little.addMessage(msg, len);
}

void Host::gotChunkTail(char * msg, Uint32 len)
{
	little.addMessage(msg, len);

	if (little.buffer)
	{
		gotMessage(little.buffer, little.currentLength);
		little.deleteMessage();
	}
}

void Host::gotBigChunk(Uint32 total, char * msg, Uint32 len)
{
	if (ftLength == 0)
	{
		ftLength = total;
	}
	else if (ftLength != total)
	{	// This protocol implementation does not support recursive/parallel transfers
		large.deleteMessage();
		ftLength = total;
	}

	large.addMessage(msg, len);

	if (large.currentLength == total)
	{
		if (large.buffer)
		{
			gotMessage(large.buffer, large.currentLength);
			large.deleteMessage();
		}
		ftLength = 0;
	}
	else if (large.currentLength > total)
	{
		large.deleteMessage();
		ftLength = 0;
	}
}

void Host::gotCluster(char * msg, Uint32 len)
{
	Uint32 index = 0;

	do	// Safe provided <specialprot.h>
	{	BYTE this_len = getByte(msg, index++);

		if ((this_len == 0) || (index + this_len > len))
		{
			return;
		}

		char * this_msg = msg + index;

		gotMessage(this_msg, this_len);

		index += this_len;
	} while (index < len);
}


//////// Core out ////////

void Host::connect()
{
	connecting = true;
	lastConnect = getTime();

	char msg[8];
	msg[0] = 0x00;
	msg[1] = 0x01;
	*(Uint32*)&msg[2] = encryption.generateKey();	// generateKey() should return the same random key every time 
	*(Uint16*)&msg[6] = 0x01;

	printf("Connecting host %s : %i\n", remote.getString(), remote.getPort());

	send(msg, 8);
}

void Host::disconnect(bool notify)
{
	if (notify)
	{
		char msg[2];
		msg[0] = 0x00;
		msg[1] = 0x07;

		send(msg, 2);
		send(msg, 2);
	}

	printf("Closing...\n");

	killMe = true;
}

void Host::syncClocks()
{
	syncing = true;
	lastSync = getTime();

	char msg[14];
	msg[0] = 0x00;
	msg[1] = 0x05;
	*(Uint32*)&msg[2] = getTime();
	*(Uint32*)&msg[6] = msgSent;
	*(Uint32*)&msg[10] = msgRecv;

	post(msg, 14, false);
}


//////// Host list ////////

hostList::hostList()
{
	maximumHosts = 0;
	socket.create(0);
}

hostList::hostList(Uint16 max_hosts, Uint16 local_port)
{
	maximumHosts = max_hosts;
	socket.create(local_port);
}

hostList::~hostList()
{
}

Host * hostList::getSource(INADDR * remote)
{
	// Look up the source
	_listnode <Host> * parse = list.head;

	while (parse)
	{
		if (parse->item->validateSource(remote))
		{
			return parse->item;
		}

		parse = parse->next;
	}

	// If possible, add it
	Host * h = NULL;

	if (list.total < maximumHosts)
	{
		h = new Host(true, remote, &socket);
		if (h) list.append(h);
	}

	return h;
}

void hostList::doEvents()
{
	UDPPacket * p = socket.poll();
	Uint32 time = GetTickCount();

	if (p)
	{
		Host * h = getSource(&p->src);

		if (h)
		{
			h->gotPacket(p->msg, p->len);
		}

		delete p;
	}

	_listnode <Host> * parse = list.head;

	while (parse)
	{
		Host * h = parse->item;
		parse = parse->next;

		if (h->killMe)
		{
			list.kill(h, false);
		}
		else
		{
			h->doEvents();
		}
	}

	time = GetTickCount() - time;
	if (time > SLOW_ITERATION)
	printf("Slow iteration warning (%i ms)\n", time);
}

void hostList::connectHost(BOT_INFO * botInfo)
{
	INADDR addr(botInfo->ip, botInfo->port);
	Host * h = new Host(false, &addr, &socket);
	if (h)
	{
		list.append(h);
		h->botInfo.set(*botInfo);
		h->connect();
	}
}

void hostList::connectHost(INADDR * remote)
{
	Host * h = new Host(false, remote, &socket);
	if (h)
	{
		list.append(h);
		h->connect();
	}
}

void hostList::connectHost(Uint32 ip, Uint16 port)
{
	INADDR addr(ip, port);
	connectHost(&addr);
}

void hostList::massDisconnect()
{
	list.clear(false);
}
